@@ -1,34 +1,70 @@
 //! Functions for interacting with an Atomic Server
 use url::Url;
 
-use crate::{errors::AtomicResult, parse::parse_json_ad_resource, Resource, Storelike};
+use crate::{
+    agents::Agent, commit::sign_message, errors::AtomicResult, parse::parse_json_ad_resource,
+    Resource, Storelike,
+};
 
 /// Fetches a resource, makes sure its subject matches.
 /// Checks the datatypes for the Values.
 /// Ignores all atoms where the subject is different.
 /// WARNING: Calls store methods, and is called by store methods, might get stuck in a loop!
-pub fn fetch_resource(subject: &str, store: &impl Storelike) -> AtomicResult<Resource> {
-    let body = fetch_body(subject, crate::parse::JSON_AD_MIME)?;
+pub fn fetch_resource(
+    subject: &str,
+    store: &impl Storelike,
+    for_agent: Option<Agent>,
+) -> AtomicResult<Resource> {
+    let body = fetch_body(subject, crate::parse::JSON_AD_MIME, for_agent)?;
     let resource = parse_json_ad_resource(&body, store)
         .map_err(|e| format!("Error parsing body of {}. {}", subject, e))?;
     Ok(resource)
 }
 
-/// Fetches a URL, returns its body
-pub fn fetch_body(url: &str, content_type: &str) -> AtomicResult<String> {
+/// Returns the various x-atomic authentication headers, includign agent signature
+pub fn get_authentication_headers(url: &str, agent: &Agent) -> AtomicResult<Vec<(String, String)>> {
+    let mut headers = Vec::new();
+    let now = crate::datetime_helpers::now().to_string();
+    let message = format!("{} {}", url, now);
+    let signature = sign_message(
+        &message,
+        agent
+            .private_key
+            .as_ref()
+            .ok_or("No private key in agent")?,
+        &agent.public_key,
+    )?;
+    headers.push(("x-atomic-public-key".into(), agent.public_key.to_string()));
+    headers.push(("x-atomic-signature".into(), signature));
+    headers.push(("x-atomic-timestamp".into(), now));
+    headers.push(("x-atomic-agent".into(), agent.subject.to_string()));
+    Ok(headers)
+}
+
+/// Fetches a URL, returns its body.
+/// Uses the store's Agent agent (if set) to sign the request.
+pub fn fetch_body(url: &str, content_type: &str, for_agent: Option<Agent>) -> AtomicResult<String> {
     if !url.starts_with("http") {
         return Err(format!("Could not fetch url '{}', must start with http.", url).into());
     }
+    if let Some(agent) = for_agent {
+        get_authentication_headers(url, &agent)?;
+    }
     let resp = ureq::get(url)
         .set("Accept", content_type)
         .timeout_read(2000)
         .call();
-    if resp.status() != 200 {
-        return Err(format!("Could not fetch url '{}'. Status: {}", url, resp.status()).into());
-    };
+    let status = resp.status();
     let body = resp
         .into_string()
-        .map_err(|e| format!("Could not parse response {}: {}", url, e))?;
+        .map_err(|e| format!("Could not parse HTTP response for {}: {}", url, e))?;
+    if status != 200 {
+        return Err(format!(
+            "Could not fetch url '{}'. Status: {}. Body: {}",
+            url, status, body
+        )
+        .into());
+    };
     Ok(body)
 }
 
@@ -50,7 +86,11 @@ pub fn fetch_tpf(
     if let Some(val) = q_value {
         url.query_pairs_mut().append_pair("value", val);
     }
-    let body = fetch_body(url.as_str(), "application/ad+json")?;
+    let body = fetch_body(
+        url.as_str(),
+        "application/ad+json",
+        store.get_default_agent().ok(),
+    )?;
     crate::parse::parse_json_ad_array(&body, store, false)
 }
 
@@ -97,7 +137,7 @@ mod test {
     #[ignore]
     fn fetch_resource_basic() {
         let store = crate::Store::init().unwrap();
-        let resource = fetch_resource(crate::urls::SHORTNAME, &store).unwrap();
+        let resource = fetch_resource(crate::urls::SHORTNAME, &store, None).unwrap();
         let shortname = resource.get(crate::urls::SHORTNAME).unwrap();
         assert!(shortname.to_string() == "shortname");
     }
