@@ -108,8 +108,12 @@ impl CollectionBuilder {
     }
 
     /// Converts the CollectionBuilder into a collection, with Members
-    pub fn into_collection(self, store: &impl Storelike) -> AtomicResult<Collection> {
-        Collection::new_with_members(store, self)
+    pub fn into_collection(
+        self,
+        store: &impl Storelike,
+        for_agent: Option<&str>,
+    ) -> AtomicResult<Collection> {
+        Collection::new_with_members(store, self, for_agent)
     }
 }
 
@@ -182,6 +186,7 @@ impl Collection {
     pub fn new_with_members(
         store: &impl Storelike,
         collection_builder: crate::collections::CollectionBuilder,
+        for_agent: Option<&str>,
     ) -> AtomicResult<Collection> {
         if collection_builder.page_size < 1 {
             return Err("Page size must be greater than 0".into());
@@ -201,12 +206,15 @@ impl Collection {
             .into_iter()
             .collect();
 
+        // Sort by subject, by default
+        subjects.sort();
+
         let mut resources = Vec::new();
         // If sorting is required or the nested resoureces are asked, we need to fetch all resources from the store.
         if collection_builder.sort_by.is_some() || collection_builder.include_nested {
             for subject in subjects.iter() {
                 // These nested resources are not fully calculated - they will be presented as -is
-                let resource = store.get_resource_extended(subject, true)?;
+                let resource = store.get_resource_extended(subject, true, for_agent)?;
                 resources.push(resource)
             }
             if let Some(sort) = &collection_builder.sort_by {
@@ -415,10 +423,60 @@ pub fn construct_collection(
         include_nested,
         include_external,
     };
-    let collection = Collection::new_with_members(store, collection_builder)?;
+    let collection = Collection::new_with_members(store, collection_builder, None)?;
     collection.add_to_resource(resource, store)
 }
 
+/// Creates a resource in the Store for a Class, for example `/documents`.
+/// Does not save it, though.
+pub fn create_collection_resource_for_class(
+    store: &impl Storelike,
+    class_subject: &str,
+) -> AtomicResult<Resource> {
+    let class = store.get_class(class_subject)?;
+
+    // Pluralize the shortname
+    let pluralized = match class.shortname.as_ref() {
+        "class" => "classes".to_string(),
+        "property" => "properties".to_string(),
+        other => format!("{}s", other),
+    };
+
+    let mut collection = CollectionBuilder::class_collection(&class.subject, &pluralized, store);
+
+    collection.sort_by = match class_subject {
+        urls::COMMIT => Some(urls::CREATED_AT.to_string()),
+        urls::CLASS | urls::PROPERTY => Some(urls::SHORTNAME.to_string()),
+        urls::COLLECTION => Some(urls::COLLECTION_VALUE.to_string()),
+        _other => None,
+    };
+
+    collection.sort_desc = match class_subject {
+        urls::COMMIT => true,
+        _other => false,
+    };
+
+    let mut collection_resource = collection.to_resource(store)?;
+
+    let drive = store
+        .get_self_url()
+        .ok_or("No self_url present in store, can't populate collections")?;
+
+    // Let the Collections collection be the top level item
+    let parent = if class.subject == urls::COLLECTION {
+        drive
+    } else {
+        format!("{}/collections", drive)
+    };
+
+    collection_resource.set_propval_string(urls::PARENT.into(), &parent, store)?;
+
+    collection_resource.set_propval_string(urls::NAME.into(), &pluralized, store)?;
+
+    // Should we use save_locally, which creates commits, or add_resource_unsafe, which is faster?
+    Ok(collection_resource)
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
@@ -441,7 +499,7 @@ mod test {
             include_nested: false,
             include_external: false,
         };
-        let collection = Collection::new_with_members(&store, collection_builder).unwrap();
+        let collection = Collection::new_with_members(&store, collection_builder, None).unwrap();
         assert!(collection.members.contains(&urls::PROPERTY.into()));
     }
 
@@ -461,7 +519,7 @@ mod test {
             include_nested: false,
             include_external: false,
         };
-        let collection = Collection::new_with_members(&store, collection_builder).unwrap();
+        let collection = Collection::new_with_members(&store, collection_builder, None).unwrap();
         assert!(collection.members.contains(&urls::PROPERTY.into()));
 
         let resource_collection = &collection.to_resource(&store).unwrap();
@@ -487,7 +545,7 @@ mod test {
             include_nested: true,
             include_external: false,
         };
-        let collection = Collection::new_with_members(&store, collection_builder).unwrap();
+        let collection = Collection::new_with_members(&store, collection_builder, None).unwrap();
         let first_resource = &collection.members_nested.clone().unwrap()[0];
         assert!(first_resource.get_subject().contains("Agent"));
 
@@ -511,7 +569,11 @@ mod test {
             .collect();
         println!("{:?}", subjects);
         let collections_collection = store
-            .get_resource_extended(&format!("{}/collections", store.get_base_url()), false)
+            .get_resource_extended(
+                &format!("{}/collections", store.get_base_url()),
+                false,
+                None,
+            )
             .unwrap();
         assert!(
             collections_collection
@@ -536,7 +598,7 @@ mod test {
         store.populate().unwrap();
 
         let collection_page_size = store
-            .get_resource_extended("https://atomicdata.dev/classes?page_size=1", false)
+            .get_resource_extended("https://atomicdata.dev/classes?page_size=1", false, None)
             .unwrap();
         assert!(
             collection_page_size
@@ -549,6 +611,7 @@ mod test {
             .get_resource_extended(
                 "https://atomicdata.dev/classes?current_page=2&page_size=1",
                 false,
+                None,
             )
             .unwrap();
         assert!(
