@@ -16,7 +16,8 @@ pub struct Endpoint {
     /// The part behind the server domain, e.g. '/versions' or '/collections'. Include the slash.
     pub path: String,
     /// The function that is called when the request matches the path
-    pub handle: fn(subject: url::Url, store: &Db) -> AtomicResult<Resource>,
+    pub handle:
+        fn(subject: url::Url, store: &Db, for_agent: Option<&str>) -> AtomicResult<Resource>,
     /// The list of properties that can be passed to the Endpoint as Query parameters
     pub params: Vec<String>,
     pub description: String,
